import sys
import os
from tkinter import W
import fitz
import cv2
import numpy as np
import yaml

from pdfcropper.utils.error import ErrorHandler

class PDF_to_PNG:
    def __init__(self, pdf_file, png_files, dpi=300, log='ignore',
    same_root=True, error='all'):
        self.log = log
        self.dpi = dpi
        self.zoom = self.dpi / 72  # zoom factor, standard dpi is 72
        self.pdf_file_fullpath = pdf_file
        self.path, self.pdf_file = os.path.split(pdf_file)
        self.png_files = png_files
        self.error_handler = ErrorHandler(error=error)

        self.load_pdf()

    
    def load_pdf(self):
        if self.log == 'all':
            print(f'Loading {self.pdf_file}')
        self.pdf_pages = fitz.open(self.pdf_file_fullpath)
        self.pdf_page_number = len(self.pdf_pages)

        if self.pdf_page_number != len(self.png_files):
            msg = f'Number of pages in pdf file ' +\
                f'{self.pdf_file_fullpath}:{self.pdf_page_number} ' +\
                f'does not match output number {len(self.png_files)}.'
            self.error_handler.send(IndexError, msg)


    def convert_pdf_pages_to_png(self):
        save_path = []
        magnify = fitz.Matrix(self.zoom, self.zoom)  # takes care of zooming
        for i, image in enumerate(self.pdf_pages):
            if self.log == 'all':
                print(f'Convert {self.pdf_file}:{i} to {self.png_files[i]}.')
            pix = image.get_pixmap(matrix=magnify)
            pix.set_dpi(self.dpi, self.dpi)
            save_path.append(os.path.join(self.path, self.png_files[i]))

            pix.save(save_path[i])

        return save_path



class ErrorHandler:
    def __init__(self, error='ignore'):
        self.error = error
    
    def send(self, error_type, msg):
        if self.error == 'ignore':
            pass
        elif self.error == 'print':
            print(msg)
        else:
            raise error_type(msg)


import os
import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont

from pdfcropper.utils.error import ErrorHandler


def crop_image(ifile, ofile, error='ignore', precrop=0):
    e = ErrorHandler(error=error)
    if not os.path.exists(ifile):
        msg = f':file:{ifile}: not found'
        e.send(FileNotFoundError, msg)

    #print(f'Cropping {input_filename} to {output_filename}')
    # Read in the image and convert to grayscale
    img = cv2.imread(ifile)
    if precrop > 0:
        img = img[:-precrop, :-precrop]  # Perform pre-cropping
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    gray = 255*(gray < 128).astype(np.uint8)  # To invert the text to white
    gray = cv2.morphologyEx(gray, cv2.MORPH_OPEN, np.ones(
        (2, 2), dtype=np.uint8))  # Perform noise filtering
    coords = cv2.findNonZero(gray)  # Find all non-zero points (text)
    x, y, w, h = cv2.boundingRect(coords)  # Find minimum spanning bounding box
    # Crop the image - note we do this on the original image
    cropped_image = img[y:y+h, x:x+w]
    if 0 in cropped_image.shape:
        msg = f':image:{ifile}: is empty'
        e.send(IndexError, msg)

    cv2.imwrite(ofile, cropped_image)


def add_text(ifile, ofile, text, padding=100):
    image = Image.open(ifile)
    top = padding
    width, height = image.size
    new_width = width
    new_height = height + top
    result = Image.new(image.mode, (new_width, new_height), (255, 255, 255))
    result.paste(image, (0, top))
    I1 = ImageDraw.Draw(result)
 
    # Custom font style and font size
    myFont = ImageFont.truetype('ariblk.ttf', 48)

    text_length = len(text)
    if text_length > 75:
        start_px = width/10
    elif text_length > 65:
        start_px = width/8
    elif text_length > 55:
        start_px = width/6
    else:
        start_px = width/4
    
    # Add Text to an image
    I1.text((width/4, 20), text, font=myFont, fill =(0, 0, 0))
    result.save(ofile)



"""
pdf-cropper.

A simple library that crops pdf and outputs png files.
"""

__version__ = "0.1.4"
__author__ = 'Ali Soltani'
__email__ = 'realalisoltani@gmail.com'
__url__ = 'https://gitlab.com/ali7line/awesome-scripts/-/tree/master/pdfcropper'
__pypi_deps__= ['pyyaml', 'opencv-python', 'numpy', 'pymupdf', 'Pillow']



import sys
import os

from pdfcropper.pdf_to_png import PDF_to_PNG
from pdfcropper.config import LoadConfig
from pdfcropper.crop_png import crop_image, add_text


def main():
    if len(sys.argv) == 2:
        cfname = sys.argv[1]
        config = LoadConfig(cfname)
    else:
        raise ValueError("invalid command line input")

    for info in config.pdf_files:
        root = config.root_path
        pdf_file = os.path.join(root, info['path'])
        png_files = info['pages']
        titles = info['titles']
        p2p = PDF_to_PNG(pdf_file, png_files)
        saved_pngs = p2p.convert_pdf_pages_to_png()
        for i, png_file in enumerate(saved_pngs):
            png_file = os.path.join(root, png_file)
            crop_image(png_file, png_file)
            add_text(png_file, png_file, titles[i])



if __name__ == '__main__':
    main()



import yaml

from pdfcropper.utils.error import ErrorHandler

class LoadConfig:
    def __init__(self, cfname):
        self.default_error = 'all'
        self.default_dpi = 300
        self.default_logfile = 'pdfcropper.log'
        self.default_replace = False
        self.default_root_path = ''
        self.default_same_root = True

        self.cfname = cfname
        self.load_config()
        self.check_config()

    def load_config(self):
        with open(self.cfname, 'r') as f:
            self.config_dict = yaml.safe_load(f)
        

    def check_config(self):
        self.error = self.config_dict.get('error', self.default_error)
        self.dpi = self.config_dict.get('dpi', self.default_dpi)
        self.logfile = self.config_dict.get('logfile', self.default_logfile)
        self.replace = self.config_dict.get('replace', self.default_replace)
        self.root_path = self.config_dict.get('root_path', self.default_root_path)
        self.same_root = self.config_dict.get('same_root', self.default_same_root)
        e = ErrorHandler(error=self.error)

        self.pdf_files = self.config_dict.get('pdf_files', None)
        if not self.pdf_files:
            msg = ':pdf_files: not defined in config file'
            e.send(KeyError, msg)


        for item_number, task in enumerate(self.pdf_files):
            path = task.get('path', None)
            if not path:
                msg = f':pdf_files:{item_number}:path: not defined in config file'
                e.send(KeyError, msg)
                
            pages = task.get('pages', None)
            if not pages:
                msg = f':pdf_files:{item_number}:pages: not defined in config file'
                e.send(KeyError, msg)

            titles = task.get('titles', None)
            if not titles:
                msg = f':titles:{item_number}: not defined in config file'
                e.send(KeyError, msg)


    def check_path(self):
        # check main_path exits
        # check pdf_files path exists
        # check write access to output location
        # check logfile write access
        pass


from setuptools import find_packages, setup
import os

from pdfcropper import __version__, __author__, __email__, __url__, __pypi_deps__

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='pdf-image-cropper',
    version=__version__,
    description='Crop pdf and conver to png',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    keywords='pdf crop png',
    author=__author__,
    author_email=__email__,
    url=__url__,
    license='MIT',
    platforms='POSIX',
    python_requires='>=3.7',
    packages=find_packages(),
    install_requires=__pypi_deps__,
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points = {
        'console_scripts': ['pdfcropper=pdfcropper.command_line:main'],
    }
)
